<?php
	// curlClass (c) Katherine Harvest, 2008
	// distributed under GPL

	define( 'COOKIEFILE', 'cookiefile' );

	if ( !defined( 'DEBUG' ) )	define( 'DEBUG',	false );

	class curlClass 
	{
		private $ch;
		private $pageText;			// text of last page loaded
		private $lastPageURL 	= 	'';	// used for referer
		private $info 		= 	null;

		// for parallel transfers (multi handle)
		private $multi;
		private	$easy_handles	=	Array();
		private $CurlData	=	Array();

		private $sameSiteReferer=	false;	// always list a referer from the same site

		public function useSameSiteReferer( $status )
		{
			$this->useSameSiteReferer 	=	$status;
		}

		function CURL_WriteFunction_Callback( $ch, $data )
		{
			$info			=	curl_getinfo( $ch, CURLINFO_EFFECTIVE_URL );

			if ( empty( $this->CurlData[ $info ] ) )
				$this->CurlData[ $info ]	=	$data;
			else
				$this->CurlData[ $info ]	.=	$data;

			return strlen( $data );
		}

		protected function	InitHandle( $url, $MethodSpecificHeaders = Array(), $usemulti = false )
		{
			$this->ch		=	curl_init();	

			curl_setopt( $this->ch, CURLOPT_URL, $url );
			curl_setopt( $this->ch, CURLOPT_HEADER, 0 );

			if ( $usemulti )
				curl_setopt( $this->ch, CURLOPT_WRITEFUNCTION, Array( 'curlClass', 'CURL_WriteFunction_Callback' ) );
			else
				curl_setopt( $this->ch, CURLOPT_RETURNTRANSFER, 1 );

			curl_setopt( $this->ch, CURLOPT_FOLLOWLOCATION, 1 );
			curl_setopt( $this->ch, CURLOPT_AUTOREFERER, 1 );
			curl_setopt( $this->ch, CURLOPT_COOKIEFILE, COOKIEFILE );
			curl_setopt( $this->ch, CURLOPT_COOKIEJAR, COOKIEFILE );
			curl_setopt( $this->ch, CURLOPT_VERBOSE, DEBUG );

			$ExtraHeaders		=	Array(				// for some reason curl uses something weird by default, even though
				'Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8',
				'Accept-Language: en-us,en;q=0.5',
				'Accept-Charset: ISO-8859-1,utf-8;q=0.7,*;q=0.7',
				'Expect:'
			);
			if ( !empty( $MethodSpecificHeaders ) )
				$ExtraHeaders	=	array_merge( $ExtraHeaders, $MethodSpecificHeaders );
			curl_setopt( $this->ch, CURLOPT_HTTPHEADER, $ExtraHeaders );

			if ( $this->sameSiteReferer )
			{
				$fakeReferer	=	dirname( $url );
				if ( empty( $this->lastPageURL ) )					// no last page in this transaction, use fake referer
					curl_setopt( $this->ch, CURLOPT_REFERER, $fakeReferer );
				else if ( substr( $this->lastPageURL, 0, strlen( $fakeReferer ) ) == $fakeReferer )
					curl_setopt( $this->ch, CURLOPT_REFERER, $this->lastPageURL );
				else
					curl_setopt( $this->ch, CURLOPT_REFERER, $fakeReferer );	// real last page isn't on same site, use fake
			}
			else
				if ( !empty( $this->lastPageURL ) )
					curl_setopt( $this->ch, CURLOPT_REFERER, $this->lastPageURL );

			$this->lastPageURL	=	$url;
		}

		private	function	ExecHandle()
		{
			$this->pageText 	=	curl_exec( $this->ch );
			$this->info 		= 	curl_getinfo( $this->ch );
			curl_close( $this->ch );
			unset( $this->ch );
		}

		private function	DoHandleAction( $usemulti )	// perform the correct handle action, whether we're using multi or not
		{
			if ( $usemulti )
			{	
				if ( ! $this->multi )
					$this->multi	=	curl_multi_init();

				$this->easy_handles[]	=	$this->ch;
				curl_multi_add_handle( $this->multi, $this->ch );
				$this->ch		=	null;
			}
			else
			{
				$this->ExecHandle();
				return $this->pageText;	
			}
		}

		public function		ExecMulti()
		{
			$active 	= 	null;
			do 
			{
			    	$mrc 	= 	curl_multi_exec( $this->multi, $active );
			} 
			while ( $mrc == CURLM_CALL_MULTI_PERFORM );

			while ( $active && $mrc == CURLM_OK ) 
			{
				if ( curl_multi_select( $this->multi ) != -1 ) 
				{
					do 
					{
						$mrc = curl_multi_exec( $this->multi, $active );
					} 
					while ( $mrc == CURLM_CALL_MULTI_PERFORM );
				}
			}

			// close all the easy handles
			foreach ( $this->easy_handles as $ch )
			{
				curl_multi_remove_handle( $this->multi, $ch );
				curl_close( $ch );
			}

			$this->easy_handles	=	Array();

			// close the multi handle
			curl_multi_close( $this->multi );	$this->multi	=	null;

			// return all our data
			$ret		=	$this->CurlData;
			$this->CurlData	=	Array();
			return $ret;
		}

		public function		GetNumQueued()
		{
			return count( $this->easy_handles );
		}

		public function		DoGet( $url, $usemulti = false )
		{
			$this->InitHandle( $url, Array(), $usemulti );
			curl_setopt( $this->ch, CURLOPT_POST, 0 );
			return $this->DoHandleAction( $usemulti );
		}

		public function		DoPost( $url, $data, $usemulti = false )
		{
			$ExtraHeaders		=	Array(				// for some reason curl uses something weird by default, even though
				'Content-Type: application/x-www-form-urlencoded'	// the docs say it uses x-www-form-urlencoded
			);

			$this->InitHandle( $url, $ExtraHeaders, $usemulti );
			curl_setopt( $this->ch, CURLOPT_POST, 1 );
			if ( is_array( $data ) )
				curl_setopt( $this->ch, CURLOPT_POSTFIELDS, http_build_query( $data ) );
			else
				curl_setopt( $this->ch, CURLOPT_POSTFIELDS, $data );

			return $this->DoHandleAction( $usemulti );
		}

		public function		GetPageDOM()
		{
			$myDOM				=	new DOMDocument;
			$myDOM->formatOutput		=	true;
			$myDOM->preserveWhitespace	=	false;

			@$myDOM->loadHTML( $this->pageText );

			return $myDOM;
		}

		public function		GetXMLDOM()
		{
			$myDOM				=	new DOMDocument;
			$myDOM->formatOutput		=	true;
			$myDOM->preserveWhitespace	=	false;

			@$myDOM->loadXML( $this->pageText );

			return $myDOM;
		}

		public function		GetHTTPCode()
		{
			if ( isset( $this->info ) && isset( $this->info[ 'http_code' ] ) )
				return $this->info[ 'http_code' ];
			else
				return null;
		}
	}

?>
